@@ -6,6 +6,7 @@ from __future__ import annotations
 import inspect
 import itertools
 import json
+import logging
 import os
 import typing as t
 from dataclasses import dataclass
@@ -36,8 +37,9 @@ from piccolo_api.session_auth.endpoints import session_login, session_logout
 from piccolo_api.session_auth.middleware import SessionsAuthBackend
 from piccolo_api.session_auth.tables import SessionsBase
 from pydantic import BaseModel, Field, ValidationError
-from starlette.exceptions import ExceptionMiddleware, HTTPException
+from starlette.middleware import Middleware
 from starlette.middleware.authentication import AuthenticationMiddleware
+from starlette.middleware.exceptions import HTTPException
 from starlette.requests import Request
 from starlette.responses import HTMLResponse, JSONResponse
 from starlette.staticfiles import StaticFiles
@@ -50,6 +52,9 @@ from .translations.models import (
 )
 from .version import __VERSION__ as PICCOLO_ADMIN_VERSION
 
+logger = logging.getLogger(__name__)
+
+
 ASSET_PATH = os.path.join(os.path.dirname(__file__), "dist")
 
 
@@ -105,15 +110,53 @@ class TableConfig:
         tag in the UI.
     :param hooks:
         These are passed directly to
-        :class:`PiccoloCRUD <piccolo_api.crud.endpoints>`, which powers Piccolo
-        Admin under the hood. It allows you to run custom logic when a row
-        is modified.
+        :class:`PiccoloCRUD <piccolo_api.crud.endpoints.PiccoloCRUD>`, which
+        powers Piccolo Admin under the hood. It allows you to run custom logic
+        when a row is modified.
     :param media_storage:
         These columns will be used to store media. We don't directly store the
         media in the database, but instead store a string, which is a unique
         identifier, and can be used to retrieve a URL for accessing the file.
         Piccolo Admin automatically renders a file upload widget for each media
         column in the UI.
+    :param validators:
+        The :class:`Validators <piccolo_api.crud.endpoints.Validators>` are
+        passed directly to
+        :class:`PiccoloCRUD <piccolo_api.crud.endpoints.PiccoloCRUD>`, which
+        powers Piccolo Admin under the hood. It allows fine grained access
+        control over each API endpoint. For example, limiting which users can
+        ``POST`` data::
+
+            from piccolo_api.crud.endpoints import PiccoloCRUD
+            from starlette.exceptions import HTTPException
+            from starlette.requests import Request
+
+
+            async def manager_only(
+                piccolo_crud: PiccoloCRUD,
+                request: Request
+            ):
+                # The Piccolo `BaseUser` can be accessed from the request.
+                user = request.user.user
+
+                # Assuming we have another database table where we record
+                # users with certain permissions.
+                manager = await Manager.exists().where(manager.user == user)
+
+                if not manager:
+                    # Raise a Starlette exception if we want to reject the
+                    # request.
+                    raise HTTPException(
+                        status_code=403,
+                        detail="Only managers are allowed to do this"
+                    )
+
+            admin = create_admin(
+                tables=TableConfig(
+                    Movie,
+                    validators=Validators(post_single=[manager_only])
+                )
+            )
 
     """
 
@@ -125,6 +168,7 @@ class TableConfig:
     rich_text_columns: t.Optional[t.List[Column]] = None
     hooks: t.Optional[t.List[Hook]] = None
     media_storage: t.Optional[t.Sequence[MediaStorage]] = None
+    validators: t.Optional[Validators] = None
 
     def __post_init__(self):
         if self.visible_columns and self.exclude_visible_columns:
@@ -282,6 +326,11 @@ def superuser_validators(piccolo_crud: PiccoloCRUD, request: Request):
         )
 
 
+async def log_error(request: Request, exc: HTTPException):
+    logger.exception("Server error")
+    raise exc
+
+
 class AdminRouter(FastAPI):
     """
     The root returns a single page app. The other URLs are REST endpoints.
@@ -307,9 +356,19 @@ class AdminRouter(FastAPI):
         site_name: str = "Piccolo Admin",
         default_language_code: str = "auto",
         translations: t.List[Translation] = None,
+        allowed_hosts: t.Sequence[str] = [],
+        debug: bool = False,
     ) -> None:
         super().__init__(
-            title=site_name, description="Piccolo API documentation"
+            title=site_name,
+            description=f"{site_name} documentation",
+            middleware=[
+                Middleware(CSRFMiddleware, allowed_hosts=allowed_hosts)
+            ],
+            debug=debug,
+            exception_handlers={500: log_error},
+            docs_url=None,
+            redoc_url=None,
         )
 
         #######################################################################
@@ -386,7 +445,12 @@ class AdminRouter(FastAPI):
 
         #######################################################################
 
-        private_app = FastAPI(docs_url=None)
+        private_app = FastAPI(
+            docs_url=None,
+            redoc_url=None,
+            debug=debug,
+            exception_handlers={500: log_error},
+        )
         private_app.mount("/docs/", swagger_ui(schema_url="../openapi.json"))
 
         for table_config in table_configs:
@@ -398,11 +462,10 @@ class AdminRouter(FastAPI):
             )
             media_columns_names = table_config.get_media_columns_names()
 
-            validators = (
-                Validators(every=[superuser_validators])
-                if table_class in (auth_table, session_table)
-                else None
-            )
+            validators = table_config.validators
+            if table_class in (auth_table, session_table):
+                validators = validators or Validators()
+                validators.every = [superuser_validators, *validators.every]
 
             FastAPIWrapper(
                 root_url=f"/tables/{table_class._meta.tablename}/",
@@ -517,7 +580,12 @@ class AdminRouter(FastAPI):
 
         #######################################################################
 
-        public_app = FastAPI(docs_url=None)
+        public_app = FastAPI(
+            redoc_url=None,
+            docs_url=None,
+            debug=debug,
+            exception_handlers={500: log_error},
+        )
         public_app.mount("/docs/", swagger_ui(schema_url="../openapi.json"))
 
         if not rate_limit_provider:
@@ -883,6 +951,7 @@ def create_admin(
     translations: t.List[Translation] = None,
     auto_include_related: bool = True,
     allowed_hosts: t.Sequence[str] = [],
+    debug: bool = False,
 ):
     """
     :param tables:
@@ -981,6 +1050,11 @@ def create_admin(
         This is used by the :class:`CSRFMiddleware <piccolo_api.csrf.middleware.CSRFMiddleware>`
         as an additional layer of protection when the admin is run under HTTPS.
         It must be a sequence of strings, such as ``['my_site.com']``.
+    :param debug:
+        If ``True``, debug mode is enabled. Any unhandled exceptions will
+        return a stack trace, rather than a generic 500 error. Don't use this
+        in production!
+
     """  # noqa: E501
     auth_table = auth_table or BaseUser
     session_table = session_table or SessionsBase
@@ -1008,24 +1082,21 @@ def create_admin(
 
         tables = all_table_classes_with_configs
 
-    return ExceptionMiddleware(
-        CSRFMiddleware(
-            AdminRouter(
-                *tables,
-                forms=forms,
-                auth_table=auth_table,
-                session_table=session_table,
-                session_expiry=session_expiry,
-                max_session_expiry=max_session_expiry,
-                increase_expiry=increase_expiry,
-                page_size=page_size,
-                read_only=read_only,
-                rate_limit_provider=rate_limit_provider,
-                production=production,
-                site_name=site_name,
-                default_language_code=default_language_code,
-                translations=translations,
-            ),
-            allowed_hosts=allowed_hosts,
-        )
+    return AdminRouter(
+        *tables,
+        forms=forms,
+        auth_table=auth_table,
+        session_table=session_table,
+        session_expiry=session_expiry,
+        max_session_expiry=max_session_expiry,
+        increase_expiry=increase_expiry,
+        page_size=page_size,
+        read_only=read_only,
+        rate_limit_provider=rate_limit_provider,
+        production=production,
+        site_name=site_name,
+        default_language_code=default_language_code,
+        translations=translations,
+        allowed_hosts=allowed_hosts,
+        debug=debug,
     )
